unit EasyReg;

{===============================================================================

  EasyReg unit - Janssen Software - June 24, 2004

  EasyReg is a simple (easy) component, to create and verify licensekeys in
  order to protect unwanted use of your software. It does NOT shut down a
  running program or show nag-screens, it ONLY verifies is a given licensekey
  is valid, and reveals the information stored within. This information
  consists of
  * a LicenseID (e.g. a customer ID)
  * a Date field (can be used to limit useage upto a specific date)
  * an array of Options (to give or deny users access to parts of your program)
  * a Version number (could be matched with the application version number)
  The licensekey is scrambled, and will be tested on the serial number of the
  first harddisk found in the PC the check routine is running on. This way, a
  licensekey is practically bound to one PC.
  The key is encoded based on an ApplicationID and encoding Sequence (for
  example '0123456789ABCDEF' for Hex encoding, but any encoding string is fine),
  to prevent others to generate keys for or decode existing keys from your
  program.

  To use EasyReg, follow these steps:
  * On the PC the software will be running on, generate a dummy key (NewDummy).
  * Let the customer e-mail or fax this dummy key to you, accompanied by a name
    to wich the application will be licensed to.
  * On your own PC, create a new licensekey with this dummy, a LicenseID, a
    name and some options. The licensekey can be saved in various ways.
  * Send the new licensekey file to your customer.
  * Let the CheckKey function verify the state of the key, and make your program
    act accordingly.
  Please note : Keep a good administration of issued keys.

  Key anatomy : LicenseID-AppID-Date-Options-Name-System-Version&CRC

  TEasyReg is freeware and donated to the public domain, but please inform
  us of any enhancements made. This unit may be freely distributed as long as it
  is accompanied by these comments, and the origin is not concealed.

  email : components@janssensoftware.com
  website : www.janssensoftware.com

===============================================================================}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TEROptions = array[0..31] of Boolean;

  TERKeyLocation = (erManual, erIni, erKey, erRegistry);

  TERName = String[40];

  TEasyReg = class(TComponent)
  private
    { Private declarations }
    FApplicationID : Word;
    FLicenseID : Word;
    FLicenseDate : TDateTime;
    FLicenseOptions : TEROptions;
    FKeyLocation : TERKeyLocation;
    FLicenseKey : String;
    FLicenseName : TERName;
    FVersion : Byte;
    FError : Word;
    FCompany : TERName;
    FProduct : TERName;
    FSystem : Word;
    FSequence : String;
    function GetOptions(Index: Integer): Boolean;
    procedure SetOptions(Index: Integer; const Value: Boolean);
    procedure SetSequence(Value: String);
    function GetErrorString : String;
    function Scramble(const Value: String): Word;
    function EncodePart(const Value: Word) : String;
    function Decode(const Key : String; const FullExtract: Boolean) : Boolean;
    function ReadKey : Boolean;
    function HDSerial : String;
    function Encode(const SystemID : Word) : Boolean;
    function SaveKey : Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    function CheckKey : Boolean;
    function NewKey(const Dummy: String) : Boolean;
    function NewDummy : String;
    property ErrorNumber : Word read FError;
    property ErrorString : String read GetErrorString;
    property LicenseID : Word read FLicenseID write FLicenseID;
    property LicenseDate : TDateTime read FLicenseDate write FLicenseDate;
    property LicenseKey : String read FLicenseKey write FLicenseKey;
    property LicenseName : TERName read FLicenseName write FLicenseName;
    property LicenseOptions[Index : Integer] : Boolean read GetOptions write SetOptions;
  published
    { Published declarations }
    property ApplicationID : Word read FApplicationID write FApplicationID;
    property KeyLocation : TERKeyLocation read FKeyLocation write FKeyLocation;
    property Company : TERName read FCompany write FCompany;
    property Product : TERName read FProduct write FProduct;
    property Version : Byte read FVersion write FVersion;
    property Sequence : String read FSequence write SetSequence;
  end;

procedure Register;

const
  erOk               =  0;
  erEmptyKey         =  1;
  erFileNotFound     =  2;
  erAppMismatch      =  3;
  erNameMismatch     =  4;
  erSystemMismatch   =  5;
  erCRCMismatch      =  6;
  erInvalidKey       =  7;
  erWriteError       =  8;
  erEmptyApplication =  9;
  erEmptyLicense     = 10;
  erEmptyName        = 11;
  erEmptyCompany     = 12;
  erEmptyProduct     = 13;
  erEmptyVersion     = 14;
  erEmptySequence    = 15;
  erDummyKey         = 16;
  erInvalidDummy     = 17;
  erInvalidCompany   = 18;
  erInvalidProduct   = 19;

implementation

uses
  IniFiles, Registry, Math;

{--- General helper functions -------------------------------------------------}

function SeqToInt(const Value, Sequence: string): Cardinal;
{ Returns the decimal value of a sequence encoded with IntToSeq }
var
  iDec : Cardinal;
  iLen, iPos, iIndex: Integer;
begin
  iDec := 0;
  iLen := Length(Value);
  for iPos := iLen downto 1 do
    begin
      iIndex := Pos(Value[iPos], Sequence) - 1;
      if (iIndex < 0) then
        begin
          iDec := 0;
          Break;
        end
      else
        iDec := iDec + Trunc(iIndex * IntPower(Length(Sequence), iLen-iPos));
    end;
  Result := iDec;
end;

function IntToSeq(const Value : Cardinal; const Sequence: String): String;
{ Encodes an integer using a custom sequence, e.g. '0123456789ABCDEF' for
  hexadecimal encoding }
var
  sStr : String;
  iTmp, iDec, iLen : Cardinal;
begin
  sStr := '';
  iDec := Value;
  iLen := Abs(Length(Sequence));
  while (iDec >= iLen) do
    begin
      iTmp := iDec mod iLen;
      sStr := Sequence[iTmp+1] + sStr;
      iDec := iDec div iLen;
    end;
  if (iDec > 0) then
    sStr := Sequence[iDec+1] + sStr;
  Result := sStr;
end;

{--- Private ------------------------------------------------------------------}

function TEasyReg.GetOptions(Index: Integer): Boolean;
begin
  if (Index >= Low(FLicenseOptions)) and (Index <= High(FLicenseOptions)) then
    Result := FLicenseOptions[Index]
  else
    raise ERangeError.CreateFmt('%d is not within the valid range of %d..%d', [Index, Low(FLicenseOptions), High(FLicenseOptions)]);
end;

procedure TEasyReg.SetOptions(Index: Integer; const Value: Boolean);
begin
  if (Index >= Low(FLicenseOptions)) and (Index <= High(FLicenseOptions)) then
    begin
      if (Value <> FLicenseOptions[Index]) then
        FLicenseOptions[Index] := Value;
    end
  else
    raise ERangeError.CreateFmt('%d is not within the valid range of %d..%d', [Index, Low(FLicenseOptions), High(FLicenseOptions)]);
end;

procedure TEasyReg.SetSequence(Value: String);
var
  i, j : Integer;
begin
  if (Value <> FSequence) then
    begin
      if (Length(Value) < 2) then
        raise Exception.CreateFmt('Sequence should contain at least 2 characters', []);
      for i := 1 to Length(Value) do
        for j := i + 1 to Length(Value) do
          if (Value[i] = Value[j]) then
            raise Exception.CreateFmt('Sequence %s contains the character %s twice', [Value, Value[i]]);
      FSequence := Value;
    end;
end;

function TEasyReg.GetErrorString : String;
{ Returns the errorcode stored in FError in a string. }
begin
  case FError of
    erOk               : Result := 'OK';
    erEmptyKey         : Result := 'Empty key';
    erFileNotFound     : Result := 'File not found';
    erAppMismatch      : Result := 'Application ID mismatch';
    erNameMismatch     : Result := 'Name mismatch';
    erSystemMismatch   : Result := 'System mismatch';
    erCRCMismatch      : Result := 'CRC mismatch';
    erInvalidKey       : Result := 'Invalid key';
    erWriteError       : Result := 'Write error';
    erEmptyApplication : Result := 'Empty application ID';
    erEmptyLicense     : Result := 'Empty license ID';
    erEmptyName        : Result := 'Empty name';
    erEmptyCompany     : Result := 'Empty company';
    erEmptyProduct     : Result := 'Empty product';
    erEmptyVersion     : Result := 'Empty version';
    erEmptySequence    : Result := 'Empty sequence';
    erDummyKey         : Result := 'Dummy key';
    erInvalidDummy     : Result := 'Invalid dummy key';
    erInvalidCompany   : Result := 'Invalid company';
    erInvalidProduct   : Result := 'Invalid product';
    else                 Result := Format('Unknown error %d', [FError])
  end; { case FError }
end;

function TEasyReg.Scramble(const Value: String): Word;
{ Calculates a checksum for any given string. }
var
  iPos, iLen, iScramble : Integer;
begin
  iScramble := LoByte(FApplicationID) * HiByte(FApplicationID) +
               LoByte(FApplicationID) + HiByte(FApplicationID);
  iLen := Length(Value);
  for iPos := 1 to iLen do
    iScramble := iScramble + (iPos * Ord(Value[iPos]));
  Result := LoWord(iScramble);
end;

function TEasyReg.EncodePart(const Value: Word) : String;
{ Encodes a word to a string. See also : DecodePart }
var
  iPos, iLen : Integer;
  sNul : Char;
begin
  sNul := FSequence[1];
  iLen := Length(IntToSeq(MAXWORD, FSequence));
  Result := IntToSeq(Value, FSequence);
  for iPos := Length(Result) + 1 to iLen do
    Result := sNul + Result;
end;

const
  erDummyName = #10 + 'EasyReg demo' + #13;

function TEasyReg.Decode(const Key : String; const FullExtract: Boolean) : Boolean;
{ Decodes a given LicenseKey. When the name matches 'erDummyName', the key is
  assumed to be a dummy key, and thus the SystemID is saved. Otherwise, the
  LicenseID, Date, Options and Version are saved in the corresponding fields. }

  function GetPart(const Key: String; const Index: Integer) : String;
  { Helper function. Extracts one part of the key. }
  var
    iLen : Integer;
  begin
    iLen   := Length(EncodePart(MAXWORD));
    Result := Copy(Key, 1 + (Index - 1) * (iLen + 1), iLen);
  end;

  function DecodePart(const Key: String; const Index: Integer; var isOK: Boolean) : Word;
  { Helper function. Decodes one part of the key to a word. See allso : EncodePart }
  var
    sPart : String;
  begin
    sPart  := GetPart(Key, Index);
    if (Length(sPart) <> Length(IntToSeq(MAXWORD, FSequence))) then
      begin
        Result := 0;
        isOK   := False;
        Exit;
      end;
    Result := SeqToInt(sPart, FSequence);
    if (Result = 0) and (sPart[1] <> FSequence[1]) then
      isOK := False;
  end;

var
  iLicAppSum, iLicID, iAppID, iLicDate, iLicOptions1, iLicOptions2, iLicName, iLicSystem, iLicVerCRC : Word;
  iVersion, iCRC : Byte;
  iOptions, iAnd : LongWord;
  i : Integer;
  isOK : Boolean;
begin
  Result := False;
  isOK   := True;
  iLicID       := DecodePart(Key, 1, isOK) Xor FApplicationID;
  iLicAppSum   := (iLicID div 2) + (FApplicationID div 2);
  iAppID       := DecodePart(Key, 2, isOK) Xor iLicID Xor iLicAppSum;
  iLicDate     := DecodePart(Key, 3, isOK) Xor iLicID;
  iLicOptions1 := DecodePart(Key, 4, isOK) Xor iLicID;
  iLicOptions2 := DecodePart(Key, 5, isOK) Xor iLicID;
  iLicName     := DecodePart(Key, 6, isOK) Xor iLicID;
  iLicSystem   := DecodePart(Key, 7, isOK) Xor iLicID;
  iLicVerCRC   := DecodePart(Key, 8, isOK) Xor iLicID;

  if (Length(Key)=0) then
    begin
      FError := erEmptyKey;
      Exit;
    end;

  if not(isOK) then
    begin
      FError := erInvalidKey;
      Exit;
    end;

  iCRC     := LoByte(iLicVerCRC);
  iVersion := HiByte(iLicVerCRC);
  if (iCRC <> LoByte(iAppID Xor iLicDate Xor iLicOptions1 Xor iLicOptions2 Xor iLicName Xor iLicSystem Xor iVersion)) then
    begin
      FError := erCRCMismatch;
      Exit;
    end;

  if (iAppID = 0) then
    begin
      FError := erEmptyApplication;
      Exit;
    end;

  if (iAppID <> FApplicationID) then
    begin
      FError := erAppMismatch;
      Exit;
    end;

  if (iVersion = 0) then
    begin
      FError := erEmptyVersion;
      Exit;
    end;

  if FullExtract then
    begin
      FLicenseDate := iLicDate;
      FVersion     := iVersion;
    end;

  if (iLicName <> Scramble(erDummyName)) and not(FullExtract) then
    { Name doesn't match 'erDummyName', but should -> Key is not a valid dummy
      key. }
    begin
      Result := False;
      FError := erInvalidDummy;
      Exit;
    end;

  if (iLicName = Scramble(erDummyName)) and not(FullExtract) then
    { Name matches 'erDummyName' -> Key is a dummy key. }
    begin
      FSystem := iLicSystem;
      Result  := True;
      Exit;
    end;

  if (iLicName = Scramble(erDummyName)) then
    { Name matches 'erDummyName', but shouldn't -> Key is a dummy key. }
    begin
      FLicenseName := '*dummy key*';
      FError       := erDummyKey;
      Exit;
    end;

  if (iLicName <> Scramble(FLicenseName)) then
    begin
      FError := erNameMismatch;
      Exit;
    end;

  if (iLicSystem <> Scramble(HDSerial)) then
    begin
      FError := erSystemMismatch;
      Exit;
    end;

  { No errors found. Save results in corresponding fields. }

  FLicenseID   := iLicID;

  iOptions := iLicOptions1;
  iOptions := (iOptions Shl 16) + iLicOptions2;
  for i := Low(TEROptions) to High(TEROptions) do
    begin
      iAnd := 1 shl i;
      FLicenseOptions[i] := (iOptions and iAnd = iAnd);
    end;
end;

function TEasyReg.ReadKey : Boolean;
{ Reads FLicenseKey and FLicenseName from the location specified in
  FKeyLocation. }
var
  sFile, sLine, sCompany, sProduct, sLicenseKey, sLicenseName, sHex, sTemp : String;
  F : TextFile;
  i, iPos, iChar : Byte;
  iXor : Integer;
begin
  Result       := False;
  FLicenseID   := 0;
  FLicenseDate := Date;
  if (FKeyLocation <> erManual) then
    begin
      FLicenseKey  := '';
      FLicenseName := '';
    end;
  for i := Low(TEROptions) to High(TEROptions) do
    FLicenseOptions[i] := False;
  case FKeyLocation of
    erManual   : ; { Info stored in FLicenseKey and FLicenseName is used }
    erIni      : begin
                   sFile := ChangeFileExt(ParamStr(0), '.ini');
                   if not(FileExists(sFile)) then
                     begin
                       FError := erFileNotFound;
                       Exit;
                     end;
                   with TIniFile.Create(sFile) do
                     begin
                       sCompany     := ReadString('License', 'Company', '');
                       sProduct     := ReadString('License', 'Product', '');
                       sLicenseKey  := ReadString('License', 'Key', '');
                       sLicenseName := ReadString('License', 'Name', '');
                       Free;
                     end;
                 end;
    erKey      : begin
                   sFile := ChangeFileExt(ParamStr(0), '.key');
                   if not(FileExists(sFile)) then
                     begin
                       FError := erFileNotFound;
                       Exit;
                     end;
                   sTemp := '';
                   iXor  := 1;
                   {$IOCHECKS OFF}
                   AssignFile(F, sFile);
                   Reset(F);
                   ReadLn(F, sCompany);
                   ReadLn(F, sProduct);
                   repeat
                     ReadLn(F, sLine);
                     for i := 1 to (Length(sLine) div 2) do
                       begin
                         iPos  := 1 + (i-1) * 2;
                         sHex  := '$' + sLine[iPos] + sLine[iPos+1];
                         iChar := StrToInt(sHex) Xor iXor;
                         if not (iChar in [13, 10]) then
                           sTemp := sTemp + Chr(iChar);
                         Inc(iXor);
                       end;
                   until Eof(F);
                   CloseFile(F);
                   {$IOCHECKS ON}
                   iPos  := Pos(#9, sTemp);
                   sLicenseName := Copy(sTemp, iPos + 1, MAXINT);
                   sLicenseKey  := Copy(sTemp, 1, iPos - 1);
                 end;
    erRegistry : begin
                   with TRegistry.Create do
                     begin
                       RootKey := HKEY_LOCAL_MACHINE;
                       sLine := Format('Software\%s\%s\License\', [FCompany, FProduct]);
                       if not(KeyExists(sLine)) then
                         begin
                           FError := erFileNotFound;
                           Free;
                           Exit;
                         end;
                       OpenKeyReadOnly(sLine);
                       sLicenseKey  := ReadString('Key');
                       sLicenseName := ReadString('Name');
                       CloseKey;
                       Free;
                     end;
                 end;
  end; { case FKeyLocation }
  if (Length(sLicenseKey)=0) then
    FError := erEmptyKey
  else
    if (Length(sLicenseName)=0) then
      FError := erEmptyName
    else
      if (sCompany <> FCompany) then
        FError := erInvalidCompany
      else
        if (sProduct <> FProduct) then
          FError := erInvalidProduct
        else
          begin
            Result       := True;
            FLicenseKey  := sLicenseKey;
            FLicenseName := sLicenseName;
          end;
end;

function TEasyReg.HDSerial : String;
{ Returns the first HD serial as a string. }
var
  iSerial, iDummy1, iDummy2: DWORD;
begin 
  Result := '';
  if GetVolumeInformation('C:\', NIL, 0, @iSerial, iDummy1, iDummy2, NIL, 0) then
    Result := Format('%.4x:%.4x', [HiWord(iSerial), LoWord(iSerial)]);
end;

function TEasyReg.Encode(const SystemID: Word) : Boolean;
{ Encodes the license fields and saves the result in FLicenseKey. }
var
  iLicAppSum, iLicID, iAppID, iLicDate, iLicOptions1, iLicOptions2, iLicName, iLicSystem, iCRCVersion : Word;
  iCRC : Byte;
  iOptions : LongWord;
  i : Integer;
begin
  Result := False;
  if (FApplicationID = 0) then
    begin
      FError := erEmptyApplication;
      Exit;
    end;
  if (FLicenseID = 0) then
    begin
      FError := erEmptyLicense;
      Exit;
    end;
  if (FLicenseName = '') then
    begin
      FError := erEmptyName;
      Exit;
    end;
  if (FVersion = 0) then
    begin
      FError := erEmptyVersion;
      Exit;
    end;

  iLicAppSum   := (FLicenseID div 2) + (FApplicationID div 2);
  iOptions     := 0;
  for i := Low(TEROptions) to High(TEROptions) do
    if (FLicenseOptions[i]) then
      iOptions := iOptions + (1 shl i);

  iLicID       := FLicenseID Xor FApplicationID;
  iAppID       := FApplicationID Xor FLicenseID Xor iLicAppSum;
  iLicDate     := Word(Trunc(FLicenseDate)) Xor FLicenseID;
  iLicOptions1 := HiWord(iOptions) Xor FLicenseID;
  iLicOptions2 := LoWord(iOptions) Xor FLicenseID;
  iLicName     := Scramble(FLicenseName) Xor FLicenseID;
  iLicSystem   := SystemID Xor FLicenseID;
  iCRC         := LoByte(FApplicationID Xor Word(Trunc(FLicenseDate)) Xor HiWord(iOptions) Xor LoWord(iOptions) Xor Scramble(FLicenseName) Xor SystemID Xor FVersion);
  iCRCVersion  := FVersion;
  iCRCVersion  := (iCRCVersion shl 8 + iCRC) Xor FLicenseID;
  FLicenseKey  := EncodePart(iLicID) + '-' +
                  EncodePart(iAppID) + '-' +
                  EncodePart(iLicDate) + '-' +
                  EncodePart(iLicOptions1) + '-' +
                  EncodePart(iLicOptions2) + '-' +
                  EncodePart(iLicName) + '-' +
                  EncodePart(iLicSystem) + '-' +
                  EncodePart(iCRCVersion);
  Result := True;
end;

function TEasyReg.SaveKey : Boolean;
{ Saves the FLicenseKey and FLicenseName fields to the location specified by
  FKeyLocation. }
var
  sFile, sLine, sChar : String;
  F : TextFile;
  i : Byte;
begin
  Result := False;
  case FKeyLocation of
    erManual   : ; { Info is allready stored in FLicenseKey and FLicenseName }
    erIni      : begin
                   sFile := ChangeFileExt(ParamStr(0), '.ini');
                   with TIniFile.Create(sFile) do
                     begin
                       WriteString('License', 'Company', FCompany);
                       WriteString('License', 'Product', FProduct);
                       WriteString('License', 'Key', FLicenseKey);
                       WriteString('License', 'Name', FLicenseName);
                       UpdateFile;
                       Free;
                     end;
                 end;
    erKey      : begin
                   sFile := ChangeFileExt(ParamStr(0), '.key');
                   if FileExists(sFile) then
                     DeleteFile(sFile);
                   {$IOCHECKS OFF}
                   AssignFile(F, sFile);
                   Rewrite(F);
                   WriteLn(F, FCompany);
                   WriteLn(F, FProduct);
                   sLine := FLicenseKey + #9 + FLicenseName;
                   for i := 1 to Length(sLine) do
                     begin
                       sChar := Format('%2x', [Ord(sLine[i]) Xor i]);
                       if (sChar[1]=' ') then sChar[1] := '0';
                       if (sChar[2]=' ') then sChar[2] := '0';
                       Write(F, sChar);
                       if (i mod 20) = 0 then
                         Write(F, #13 + #10);
                     end;
                   CloseFile(F);
                   {$IOCHECKS ON}
                   if (IOResult <> 0) then
                     FError := erWriteError
                 end;
    erRegistry : begin
                   if (FCompany = '') then
                     begin
                       FError := erEmptyCompany;
                       Exit;
                     end;
                   if (FProduct = '') then
                     begin
                       FError := erEmptyProduct;
                       Exit;
                     end;
                   sFile := ChangeFileExt(ParamStr(0), '.reg');
                   if FileExists(sFile) then
                     DeleteFile(sFile);
                   {$IOCHECKS OFF}
                   AssignFile(F, sFile);
                   Rewrite(F);
                   WriteLn(F, 'REGEDIT4');
                   WriteLn(F, '');
                   WriteLn(F, Format('[HKEY_LOCAL_MACHINE\Software\%s\%s\License]', [FCompany, FProduct]));
                   WriteLn(F, Format('"key"="%s"', [FLicenseKey]));
                   WriteLn(F, Format('"name"="%s"', [FLicenseName]));
                   WriteLn(F, '');
                   CloseFile(F);
                   {$IOCHECKS ON}
                   if (IOResult <> 0) then
                     FError := erWriteError
                 end;
  end; { case FKeyLocation }
  Result := (FError = erOk);
end;

{--- Protected ----------------------------------------------------------------}

{--- Public -------------------------------------------------------------------}

constructor TEasyReg.Create(AOwner: TComponent);
{ Creator sets some default values. }
begin
  inherited Create(AOwner);
  Randomize;
  FKeyLocation   := erManual;
  FCompany       := 'JanssenSoftware';
  FProduct       := 'EasyReg';
  FApplicationID := 17100;
  FVersion       := 1;
  FSequence      := '0123456789ABCDEF';
end;

function TEasyReg.CheckKey : Boolean;
{ True if read correctly, False if not (see FError for details) }
begin
  Result := False;
  FError := erOk;
  if ReadKey then
    Result := Decode(FLicenseKey, True);
end;

function TEasyReg.NewKey(const Dummy: String) : Boolean;
{ True if saved correctly, False if not (see FError for details) }
begin
  Result := False;
  FError := erOk;
  FLicenseKey := '';
  if Decode(Dummy, False) then
    if Encode(FSystem) then
      Result := SaveKey;
end;

function TEasyReg.NewDummy : String;
{ Generates a dummy key, needed as a basis for the NewKey function. }
var
  i : Byte;
begin
  Result       := '';
  FError       := erOk;
  FLicenseID   := (MAXWORD div 2) + Random(MAXWORD div 2);
  FLicenseDate := Date;
  FLicenseName := erDummyName;
  for i := Low(TEROptions) to High(TEROptions) do
    FLicenseOptions[i] := (Random(2) = 1);
  if Encode(Scramble(HDSerial)) then
    Result := FLicenseKey;
  FLicenseID   := 0;
  FLicenseKey  := '';
  FLicenseDate := Date;
  FLicenseName := '';
  for i := Low(TEROptions) to High(TEROptions) do
    FLicenseOptions[i] := False;
end;

{--- Register -----------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Samples', [TEasyReg]);
end;

end.
